@@ -1,4 +1,4 @@
-// $Id: globals.cpp,v 1.3 2000/09/11 07:46:32 eldamitri Exp $
+// $Id: globals.cpp,v 1.6 2002/07/02 22:12:57 t1mpy Exp $
 
 // id3lib: a C++ library for creating and manipulating id3v1/v2 tags
 // Copyright 1999, 2000  Scott Thomas Haug
@@ -24,7 +24,7 @@
 // id3lib.  These files are distributed with id3lib at
 // http://download.sourceforge.net/id3lib/
 
-#include "globals.h"
+#include "id3/globals.h" //has <stdlib.h> "id3/sized_types.h"
 
 #if defined HAVE_CONFIG_H
 #include <config.h>
@@ -36,7 +36,6 @@ extern "C"
 #endif
 
   const char * const ID3LIB_NAME           = _ID3LIB_NAME;
-  const char * const ID3LIB_DATE           = _ID3LIB_DATE;
   const char * const ID3LIB_VERSION        = _ID3LIB_VERSION;
   const char * const ID3LIB_FULL_NAME      = _ID3LIB_FULLNAME;
   const int          ID3LIB_MAJOR_VERSION  = _ID3LIB_MAJOR_VERSION;
@@ -48,3 +47,4 @@ extern "C"
 #ifdef _cplusplus
 }
 #endif
+
