@@ -1,4 +1,4 @@
-// $Id: tag_parse.cpp,v 1.44 2002/07/31 16:41:11 t1mpy Exp $
+// $Id: tag_parse.cpp,v 1.47 2002/11/24 17:33:30 t1mpy Exp $
 
 // id3lib: a C++ library for creating and manipulating id3v1/v2 tags
 // Copyright 1999, 2000  Scott Thomas Haug
@@ -42,50 +42,50 @@ using namespace dami;
 namespace
 {
   bool parseFrames(ID3_TagImpl& tag, ID3_Reader& rdr)
-  { 
+  {
     ID3_Reader::pos_type beg = rdr.getCur();
     io::ExitTrigger et(rdr, beg);
     ID3_Reader::pos_type last_pos = beg;
-    size_t totalSize = 0; 
-    size_t frameSize = 0; 
+    size_t totalSize = 0;
+    size_t frameSize = 0;
     while (!rdr.atEnd() && rdr.peekChar() != '\0')
-    { 
+    {
       ID3D_NOTICE( "id3::v2::parseFrames(): rdr.getBeg() = " << rdr.getBeg() );
       ID3D_NOTICE( "id3::v2::parseFrames(): rdr.getCur() = " << rdr.getCur() );
       ID3D_NOTICE( "id3::v2::parseFrames(): rdr.getEnd() = " << rdr.getEnd() );
       last_pos = rdr.getCur();
-      ID3_Frame* f = new ID3_Frame; 
+      ID3_Frame* f = new ID3_Frame;
       f->SetSpec(tag.GetSpec());
       bool goodParse = f->Parse(rdr);
       frameSize = rdr.getCur() - last_pos;
       ID3D_NOTICE( "id3::v2::parseFrames(): frameSize = " << frameSize );
       totalSize += frameSize;
-      
+
       if (frameSize == 0)
-      { 
-        // There is a problem. 
-        // If the frame size is 0, then we can't progress. 
+      {
+        // There is a problem.
+        // If the frame size is 0, then we can't progress.
         ID3D_WARNING( "id3::v2::parseFrames(): frame size is 0, can't " <<
                       "continue parsing frames");
-        delete f; 
-        // Break for now. 
-        break; 
-      } 
-      else if (!goodParse) 
-      { 
+        delete f;
+        // Break for now.
+        break;
+      }
+      else if (!goodParse)
+      {
         // bad parse!  we can't attach this frame.
         ID3D_WARNING( "id3::v2::parseFrames(): bad parse, deleting frame");
-        delete f; 
-      } 
-      else if (f->GetID() != ID3FID_METACOMPRESSION) 
-      { 
+        delete f;
+      }
+      else if (f->GetID() != ID3FID_METACOMPRESSION)
+      {
         ID3D_NOTICE( "id3::v2::parseFrames(): attaching non-compressed " <<
                      "frame");
-        // a good, uncompressed frame.  attach away! 
-        tag.AttachFrame(f); 
-      } 
-      else 
-      { 
+        // a good, uncompressed frame.  attach away!
+        tag.AttachFrame(f);
+      }
+      else
+      {
         ID3D_NOTICE( "id3::v2::parseFrames(): parsing ID3v2.2.1 " <<
                      "compressed frame");
         // hmm.  an ID3v2.2.1 compressed frame.  It contains 1 or more
@@ -95,14 +95,14 @@ namespace
         {
           ID3_MemoryReader mr(fld->GetRawBinary(), fld->BinSize());
           ID3_Reader::char_type ch = mr.readChar();
-          if (ch != 'z') 
-          { 
-            // unknown compression method 
+          if (ch != 'z')
+          {
+            // unknown compression method
             ID3D_WARNING( "id3::v2::parseFrames(): unknown compression id " <<
                           " = '" << ch << "'" );
-          } 
-          else 
-          { 
+          }
+          else
+          {
             uint32 newSize = io::readBENumber(mr, sizeof(uint32));
             size_t oldSize = f->GetDataSize() - sizeof(uint32) - 1;
             io::CompressedReader cr(mr, newSize);
@@ -119,30 +119,30 @@ namespace
         delete f;
       }
       et.setExitPos(rdr.getCur());
-    } 
+    }
     if (rdr.peekChar() == '\0')
     {
-      ID3D_NOTICE( "id3::v2::parseFrames: done parsing, padding at postion " << 
+      ID3D_NOTICE( "id3::v2::parseFrames: done parsing, padding at postion " <<
                    rdr.getCur() );
     }
     else
     {
-      ID3D_NOTICE( "id3::v2::parseFrames: done parsing, [cur, end] = [" << 
+      ID3D_NOTICE( "id3::v2::parseFrames: done parsing, [cur, end] = [" <<
                    rdr.getCur() << ", " << rdr.getEnd() << "]" );
     }
     return true;
   }
 };
- 
+
 bool id3::v2::parse(ID3_TagImpl& tag, ID3_Reader& reader)
 {
   ID3_Reader::pos_type beg = reader.getCur();
   io::ExitTrigger et(reader);
-  
+
   ID3_TagHeader hdr;
 
   io::WindowedReader wr(reader, ID3_TagHeader::SIZE);
-  
+
   if (!hdr.Parse(wr) || wr.getCur() == beg)
   {
     ID3D_NOTICE( "id3::v2::parse(): parsing header failes" );
@@ -171,8 +171,8 @@ bool id3::v2::parse(ID3_TagImpl& tag, ID3_Reader& reader)
   }
   else
   {
-    // The buffer has been unsynced.  It will have to be resynced to be 
-    // readable.  This has to be done a character at a time.  
+    // The buffer has been unsynced.  It will have to be resynced to be
+    // readable.  This has to be done a character at a time.
     //
     // The original reader may be reading in characters from a file.  Doing
     // this a character at a time is quite slow.  To improve performance, read
@@ -190,9 +190,9 @@ bool id3::v2::parse(ID3_TagImpl& tag, ID3_Reader& reader)
     ID3D_NOTICE( "ID3_TagImpl::Parse(ID3_Reader&): unsync end = " << ur.getEnd() );
 
     // Now read the UnsyncedReader into another string, and parse the frames
-    // from the string.  This is done so that 1. the unsynced reader is 
+    // from the string.  This is done so that 1. the unsynced reader is
     // unsynced exactly once, removing the possibility of multiple unsyncings
-    // of the same string, and 2) so that calls to readChars aren't done a 
+    // of the same string, and 2) so that calls to readChars aren't done a
     // character at a time for every call
     BString synced = io::readAllBinary(ur);
     io::BStringReader sr(synced);
@@ -205,18 +205,27 @@ bool id3::v2::parse(ID3_TagImpl& tag, ID3_Reader& reader)
 void ID3_TagImpl::ParseFile()
 {
   ifstream file;
-  size_t mp3_core_size;
   if (ID3E_NoError != openReadableFile(this->GetFileName(), file))
   {
     // log this...
     return;
   }
   ID3_IFStreamReader ifsr(file);
-  io::WindowedReader wr(ifsr);
+  ParseReader(ifsr);
+  file.close();
+}
+
+//used for streaming media
+void ID3_TagImpl::ParseReader(ID3_Reader &reader)
+{
+  size_t mp3_core_size;
+  size_t bytes_till_sync;
+
+  io::WindowedReader wr(reader);
   wr.setBeg(wr.getCur());
 
   _file_tags.clear();
-  _file_size = getFileSize(file);
+  _file_size = reader.getEnd();
 
   ID3_Reader::pos_type beg  = wr.getBeg();
   ID3_Reader::pos_type cur  = wr.getCur();
@@ -238,8 +247,94 @@ void ID3_TagImpl::ParseFile()
       wr.setBeg(cur);
     } while (!wr.atEnd() && cur > last);
   }
-
+  // add silly padding outside the tag to _prepended_bytes
+  if (!wr.atEnd() && wr.peekChar() == '\0')
+  {
+    ID3D_NOTICE( "ID3_TagImpl::ParseReader(): found padding outside tag" );
+    do
+    {
+      last = cur;
+      cur = wr.getCur() + 1;
+      wr.setBeg(cur);
+      wr.setCur(cur);
+    } while (!wr.atEnd() &&  cur > last && wr.peekChar() == '\0');
+  }
+  if (!wr.atEnd() && _file_size - (cur - beg) > 4 && wr.peekChar() == 255)
+  { //unfortunatly, this is necessary for finding an invalid padding
+    wr.setCur(cur + 1); //cur is known by peekChar
+    if (wr.readChar() == '\0' && wr.readChar() == '\0' && wr.peekChar() == '\0')
+    { //three empty bytes found, enough for me, this is stupid padding
+      cur += 3; //those are now allready read in (excluding the peekChar, since it will be added by do{})
+      do
+      {
+        last = cur;
+        cur = wr.getCur() + 1;
+        wr.setBeg(cur);
+        wr.setCur(cur);
+      } while (!wr.atEnd() &&  cur > last && wr.peekChar() == '\0');
+    }
+    else
+      wr.setCur(cur);
+  }
   _prepended_bytes = cur - beg;
+  // go looking for the first sync byte to add to bytes_till_sync
+  // by not adding it to _prepended_bytes, we preserve this 'unknown' data
+  // The routine's only effect is helping the lib to find things as bitrate etc.
+  beg  = wr.getBeg();
+  if (!wr.atEnd() && wr.peekChar() != 0xFF) //no sync byte, so, either this is not followed by a mp3 file or it's a fLaC file, or an encapsulating format, better check it
+  {
+    ID3D_NOTICE( "ID3_TagImpl::ParseReader(): Didn't find mp3 sync byte" );
+    if ((_file_size - (cur - beg)) >= 4)
+    { //there is room to search for some kind of ID
+      unsigned char buf[5];
+      wr.readChars(buf, 4);
+      buf[4] = '\0';
+      // check for RIFF (an encapsulating format) ID
+      if (strncmp((char*)buf, "RIFF", 4) == 0 || strncmp((char*)buf, "RIFX", 4) == 0)
+      {
+        // next 4 bytes are RIFF size, skip them
+        cur = wr.getCur() + 4;
+        wr.setCur(cur);
+        // loop until first possible sync byte
+        if (!wr.atEnd() && wr.peekChar() != 0xFF)
+        {
+          do
+          {
+            last = cur;
+            cur = wr.getCur() + 1;
+            wr.setCur(cur);
+          } while (!wr.atEnd() &&  cur > last && wr.peekChar() != 0xFF);
+        }
+      }
+      else if (strncmp((char*)buf, "fLaC", 4) == 0)
+      { //a FLAC file, no need looking for a sync byte
+        beg = cur;
+      }
+      else
+      { //since we set the cursor 4 bytes ahead for looking for RIFF, RIFX or fLaC, better set it back
+        // but peekChar allready checked the first one, so we add one
+        cur = cur + 1;
+        wr.setCur(cur);
+        //go looking for a sync byte
+        if (!wr.atEnd() && wr.peekChar() != 0xFF) //no sync byte, we have an unknown byte
+        {
+          do
+          {
+            last = cur;
+            cur = wr.getCur() + 1;
+            wr.setCur(cur);
+          } while (!wr.atEnd() &&  cur > last && wr.peekChar() != 0xFF);
+        }
+      }
+    } //if ((_file_size - (cur - beg)) >= 4)
+    else
+    { //remaining size is smaller than 4 bytes, can't be useful, but leave it for now
+      beg = cur;
+      //file.close();
+      //return;
+    }
+  }
+  bytes_till_sync = cur - beg;
 
   cur = wr.setCur(end);
   if (_file_size > _prepended_bytes)
@@ -247,35 +342,45 @@ void ID3_TagImpl::ParseFile()
     do
     {
       last = cur;
-      ID3D_NOTICE( "ID3_TagImpl::ParseFile(): beg = " << wr.getBeg() );
-      ID3D_NOTICE( "ID3_TagImpl::ParseFile(): cur = " << wr.getCur() );
-      ID3D_NOTICE( "ID3_TagImpl::ParseFile(): end = " << wr.getEnd() );
+      ID3D_NOTICE( "ID3_TagImpl::ParseReader(): beg = " << wr.getBeg() );
+      ID3D_NOTICE( "ID3_TagImpl::ParseReader(): cur = " << wr.getCur() );
+      ID3D_NOTICE( "ID3_TagImpl::ParseReader(): end = " << wr.getEnd() );
       // ...then the tags at the end
-      ID3D_NOTICE( "ID3_TagImpl::ParseFile(): musicmatch? cur = " << wr.getCur() );
+      ID3D_NOTICE( "ID3_TagImpl::ParseReader(): musicmatch? cur = " << wr.getCur() );
       if (_tags_to_parse.test(ID3TT_MUSICMATCH) && mm::parse(*this, wr))
       {
-        ID3D_NOTICE( "ID3_TagImpl::ParseFile(): musicmatch! cur = " << wr.getCur() );
+        ID3D_NOTICE( "ID3_TagImpl::ParseReader(): musicmatch! cur = " << wr.getCur() );
         _file_tags.add(ID3TT_MUSICMATCH);
         wr.setEnd(wr.getCur());
       }
-      ID3D_NOTICE( "ID3_TagImpl::ParseFile(): lyr3v1? cur = " << wr.getCur() );
+      ID3D_NOTICE( "ID3_TagImpl::ParseReader(): lyr3v1? cur = " << wr.getCur() );
       if (_tags_to_parse.test(ID3TT_LYRICS3) && lyr3::v1::parse(*this, wr))
       {
-        ID3D_NOTICE( "ID3_TagImpl::ParseFile(): lyr3v1! cur = " << wr.getCur() );
+        ID3D_NOTICE( "ID3_TagImpl::ParseReader(): lyr3v1! cur = " << wr.getCur() );
         _file_tags.add(ID3TT_LYRICS3);
         wr.setEnd(wr.getCur());
       }
-      ID3D_NOTICE( "ID3_TagImpl::ParseFile(): lyr3v2? cur = " << wr.getCur() );
+      ID3D_NOTICE( "ID3_TagImpl::ParseReader(): lyr3v2? cur = " << wr.getCur() );
       if (_tags_to_parse.test(ID3TT_LYRICS3V2) && lyr3::v2::parse(*this, wr))
       {
-        ID3D_NOTICE( "ID3_TagImpl::ParseFile(): lyr3v2! cur = " << wr.getCur() );
-        _file_tags.add(ID3TT_ID3V1);
-        wr.setEnd(wr.getCur());
+        ID3D_NOTICE( "ID3_TagImpl::ParseReader(): lyr3v2! cur = " << wr.getCur() );
+        _file_tags.add(ID3TT_LYRICS3V2);
+        cur = wr.getCur();
+        wr.setCur(wr.getEnd());//set to end to seek id3v1 tag
+        //check for id3v1 tag and set End accordingly
+        ID3D_NOTICE( "ID3_TagImpl::ParseReader(): id3v1? cur = " << wr.getCur() );
+        if (_tags_to_parse.test(ID3TT_ID3V1) && id3::v1::parse(*this, wr))
+        {
+          ID3D_NOTICE( "ID3_TagImpl::ParseReader(): id3v1! cur = " << wr.getCur() );
+          _file_tags.add(ID3TT_ID3V1);
+        }
+        wr.setCur(cur);
+        wr.setEnd(cur);
       }
-      ID3D_NOTICE( "ID3_TagImpl::ParseFile(): id3v1? cur = " << wr.getCur() );
+      ID3D_NOTICE( "ID3_TagImpl::ParseReader(): id3v1? cur = " << wr.getCur() );
       if (_tags_to_parse.test(ID3TT_ID3V1) && id3::v1::parse(*this, wr))
       {
-        ID3D_NOTICE( "ID3_TagImpl::ParseFile(): id3v1! cur = " << wr.getCur() );
+        ID3D_NOTICE( "ID3_TagImpl::ParseReader(): id3v1! cur = " << wr.getCur() );
         wr.setEnd(wr.getCur());
         _file_tags.add(ID3TT_ID3V1);
       }
@@ -284,19 +389,19 @@ void ID3_TagImpl::ParseFile()
     _appended_bytes = end - cur;
 
     // Now get the mp3 header
-    mp3_core_size = (_file_size - _appended_bytes) - _prepended_bytes;
+    mp3_core_size = (_file_size - _appended_bytes) - (_prepended_bytes + bytes_till_sync);
     if (mp3_core_size >= 4)
     { //it has at least the size for a mp3 header (a mp3 header is 4 bytes)
-      wr.setBeg(_prepended_bytes);
-      wr.setCur(_prepended_bytes);
+      wr.setBeg(_prepended_bytes + bytes_till_sync);
+      wr.setCur(_prepended_bytes + bytes_till_sync);
       wr.setEnd(_file_size - _appended_bytes);
 
       _mp3_info = new Mp3Info;
-      ID3D_NOTICE( "ID3_TagImpl::ParseFile(): mp3header? cur = " << wr.getCur() );
+      ID3D_NOTICE( "ID3_TagImpl::ParseReader(): mp3header? cur = " << wr.getCur() );
 
       if (_mp3_info->Parse(wr, mp3_core_size))
       {
-        ID3D_NOTICE( "ID3_TagImpl::ParseFile(): mp3header! cur = " << wr.getCur() );
+        ID3D_NOTICE( "ID3_TagImpl::ParseReader(): mp3header! cur = " << wr.getCur() );
       }
       else
       {
@@ -307,6 +412,5 @@ void ID3_TagImpl::ParseFile()
   }
   else
     this->SetPadding(false); //no need to pad an empty file
-  file.close();
 }
 
